﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using System.Threading;

namespace AsynchronousWPFApplication.Models
{
    public class MyViewModel : INotifyPropertyChanged
    {
        #region Properties
        private bool _isBusy;
        private int _loadingPercentage { get; set; }
        private readonly BackgroundWorker _worker;

        public string LoadingStep { get; set; }
        public List<People> Peoples { get; set; }

        public int LoadingPercentage
        {
            get { return _loadingPercentage; }
            private set
            {
                if (_loadingPercentage != value)
                {
                    _loadingPercentage = value;
                    OnPropertyChanged("LoadingPercentage");
                }
            }
        }

        public bool IsBusy
        {
            get { return _isBusy; }
            private set
            {
                if (_isBusy != value)
                {
                    _isBusy = value;
                    OnPropertyChanged("IsBusy");
                }
            }
        }
        #endregion


        public MyViewModel()
        {
            _isBusy = false;
            LoadingStep = "";
            Peoples = new List<People>();
            _worker = new BackgroundWorker();
            _worker.DoWork += new DoWorkEventHandler(_worker_DoWork);
            _worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(_worker_RunWorkerCompleted);
            _worker.ProgressChanged += new ProgressChangedEventHandler(_worker_ProgressChanged);
            _worker.WorkerSupportsCancellation = true;
            _worker.WorkerReportsProgress = true;
        }

        #region Asynchronous Loading
        private void _worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            LoadingStep = e.UserState.ToString();
            LoadingPercentage = e.ProgressPercentage;
        }

        private void _worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            IsBusy = false;
        }

        private void _worker_DoWork(object sender, DoWorkEventArgs e)
        {
            try
            {
                
                //-- Simulate big datas loading
                _worker.ReportProgress(0, "0/5 loaded");

                Peoples.Add(new People() { FirstName = "Jean" });
                Thread.Sleep(500);
                _worker.ReportProgress(20, "1/5 loaded");

                Peoples.Add(new People() { FirstName = "Aurélie" });
                Thread.Sleep(500);
                _worker.ReportProgress(40,"2/5 loaded");

                Peoples.Add(new People() { FirstName = "Bastien" });
                Thread.Sleep(500);
                _worker.ReportProgress(60, "3/5 loaded");

                Peoples.Add(new People() { FirstName = "Fiona" });
                Thread.Sleep(500);
                _worker.ReportProgress(80, "4/5 loaded");

                Peoples.Add(new People() { FirstName = "Karl" });
                Thread.Sleep(500);
                _worker.ReportProgress(100, "5/5 loaded");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }
        #endregion

        public void LoadPeople()
        {
            IsBusy = true;
            Peoples.Clear();
            _worker.RunWorkerAsync();
        }


        #region Notified Properties

        #endregion

        #region INotifyPropertyChanged Members
        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion
    }
}
